# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 feilong.me All rights reserved.
#
# @author: Felinx Lee <felinx.lee@gmail.com>
# Created on May 4, 2013
#

import logging
from tornado.options import options, define
from nsqworker import Worker

define("demoname", "demo")

logger = logging.getLogger(__name__)


class DemoPageviewWorker(Worker):
    def demo_handler(self, message):
        logger.info("%s: %s/%s, clicks %s", options.demoname,
                     options.topic, options.channel, message.body.get("clicks", 1))

        return True



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 feilong.me All rights reserved.
#
# @author: Felinx Lee <felinx.lee@gmail.com>
# Created on May 4, 2013
#

from tornado.options import define, options

from nsqworker.initiator import Initiator

define("topic", default="demo", help="nsq topic")
define("topic_prefix", default="", help="nsq topic prefix")
define("channel", default="pageview", help="nsq topic channel")
define(
    "nsq_max_processed_messages_queue",
    default=200,
    type=int,
    help="nsq max processed messages queue")
define(
    "nsqlookupd_http_addresses",
    default="http://127.0.0.1:4161",
    multiple=True,
    help="nsqlookupd http addresses")
define("debug", default=False)


def run(workers_module="nsqworker.workers", **kw):
    initiator = Initiator(
        options.topic,
        options.channel,
        topic_prefix=options.topic_prefix,
        nsq_max_processed_messages_queue=options.
        nsq_max_processed_messages_queue,
        nsqlookupd_http_addresses=options.nsqlookupd_http_addresses)
    initiator.run(workers_module=workers_module, debug=options.debug, **kw)


if __name__ == "__main__":
    run()


# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 feilong.me All rights reserved.
#
# @author: Felinx Lee <felinx.lee@gmail.com>
# Created on May 4, 2013
#
from nsqworker.initiator import Initiator
from nsqworker.workers.worker import Worker

__version__ = '0.2.10'



# -*- coding: utf-8 -*-

import logging

import nsq
from nsqworker.workers.worker import load_worker
from six import iteritems
from tornado import autoreload

logger = logging.getLogger(__name__)


class Initiator(object):
    def __init__(self,
                 topic,
                 channel,
                 topic_prefix="",
                 nsq_max_processed_messages_queue=200,
                 nsqlookupd_http_addresses="http://127.0.0.1:4161"):
        self._topic = topic
        self._channel = channel
        self._topic_prefix = topic_prefix
        self._nsq_max_processed_messages_queue = nsq_max_processed_messages_queue
        self._nsqlookupd_http_addresses = nsqlookupd_http_addresses

    def run(self, workers_module="nsqworker.workers", debug=False, **kw):
        worker = load_worker(self._topic, self._channel, workers_module)
        logger.debug("Starting worker: %s" % worker)
        topic = "%s%s" % (self._topic_prefix, self._topic)

        legacy = False
        try:
            from nsq import LegacyReader
        except ImportError:
            legacy = True

        if not legacy:
            # nsq 0.5+
            for name, handler in iteritems(worker.handlers):
                r = nsq.Reader(
                    topic,
                    "%s_%s" % (self._channel, name[0:-len("_handler")]),
                    message_handler=handler,
                    lookupd_http_addresses=self._nsqlookupd_http_addresses,
                    **kw)

                # override default preprocess and validate method with worker's
                # method
                r.validate_message = worker.validate_message
                r.preprocess_message = worker.preprocess_message
        else:
            # nsq 0.4
            r = nsq.Reader(
                all_tasks=worker.handlers,
                topic=topic,
                channel=self._channel,
                lookupd_http_addresses=self._nsqlookupd_http_addresses,
                **kw)

            r.validate_message = worker.validate_message
            r.preprocess_message = worker.preprocess_message

        if debug:
            autoreload.start()

        nsq.run()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 feilong.me All rights reserved.
#
# @author: Felinx Lee <felinx.lee@gmail.com>
# Created on May 4, 2013
#

import time
import os
import sys
import traceback
import logging

from tornado import escape

from nsqworker.compat import import_module

_nsq_processed_messages_queues = {}

logger = logging.getLogger(__name__)


def load_worker(target_topic, target_channel, workers_module):
    """Load worker according to target_topic and target_channel.

     It only loads worker from file named as {topic}/{topic}_{channel}_worker.py,
     eg. demo/apiview_pageview_worker.py, the other files will be ignored.

    """
    # Preload all of workers
    mod = import_module(workers_module)
    workers_dir = os.path.dirname(os.path.abspath(mod.__file__))

    for d in os.listdir(workers_dir):
        dir_ = os.path.join(workers_dir, d)
        if os.path.isdir(dir_):
            topic = d
            channels = []
            for f in os.listdir(dir_):
                if not f.endswith("_worker.py"):
                    continue
                if not f.startswith('_') and f.endswith('.py'):
                    # topic_channel_worker
                    channels.append(f[len(topic) + 1:-len("_worker.py")])

            for channel in channels:
                try:
                    # preload
                    _load_worker(topic, channel, workers_module)
                except Exception as e:
                    logger.warning(e)
                    logger.warning(traceback.format_exc())

    try:
        worker, name = _load_worker(target_topic, target_channel,
                                    workers_module)
        if worker:
            return worker()
        else:
            raise ImportError("%s not found!" % name)
    except ImportError as e:
        logger.error(traceback.format_exc())
        raise e


class Worker(object):

    def preprocess_message(self, message):
        """JSON decode preprocess

        An optional callable that can alter the message data before other
        task functions are called.

        We assume that the message body is a JSON here.

        """
        try:
            message.body = escape.json_decode(escape.utf8(message.body))
        except ValueError:
            logger.error("Invalid JSON: %s", message.body)

        return message

    def validate_message(self, message):
        """An optional callable that returns a boolean as to weather or not
        this message should be processed.

        """
        if isinstance(message.body, dict):
            return True
        else:
            return False

    @property
    def handlers(self):
        _handlers = {}
        for func in dir(self):
            if func.endswith("_handler"):
                _handlers[func] = _handler(self, getattr(self, func), error_callback=self.handle_error)

        return _handlers

    def handle_error(self, exec_info=None):
        """
        Overried this method to perform error handling.
        """
        pass


def _handler(self, func, error_callback=None):
    """Worker handler decorator"""
    def wrapper(message):
        logger.debug("Raw message: %s, %s", message.id, message.body)
        try:
            now = time.time()
            r = func(message)
            logger.info(
                "Task elapse(%s-%s): %s", self.__class__.__name__,
                func.__name__, time.time() - now)
            if r is None:
                return True  # True by default
        except Exception as e:
            logger.debug(
                "Error message: %s, %s", message.id, message.body)
            exc_info = sys.exc_info()
            logger.error(e, exc_info=exc_info)
            if error_callback is not None:
                error_callback(exc_info=exc_info)

        return True

    return wrapper


def _load_worker(topic, channel, workers_module):
    mod_name = "%s_%s_worker" % (topic, channel)
    pkg = "%s.%s" % (workers_module, topic)
    mod = import_module(".%s" % mod_name, package=pkg)

    # topic_channel_worker to TopicChannelWorker
    name = "".join("%s%s" % (n[0].upper(), n[1:]) for n in mod_name.split("_"))
    worker = getattr(mod, name, None)

    return worker, name



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 feilong.me All rights reserved.
#
# @author: Felinx Lee <felinx.lee@gmail.com>
# Created on May 4, 2013
#

import distutils.core
try:
    import setuptools
except ImportError:
    pass

version = "0.2.10"

distutils.core.setup(
    name="nsqworker",
    version=version,
    packages=["nsqworker", "nsqworker.workers", "nsqworker.workers.demo"],
    author="Felinx Lee",
    author_email="felinx.lee@gmail.com",
    url="https://github.com/felinx/nsqworker",
    license="http://www.apache.org/licenses/LICENSE-2.0",
    description="nsqworker is a skeletal code of one way to implement a nsq consumer in Python base on pynsq",
    install_requires=[
        "six",
    ],
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ]
)
